## Import

**npm**:
```javascript
$ npm i anabolicset
import { AnabolicSet } from 'anabolicset/src/anabolicset'
```

**cdn**:
```html
<script src="https://cdn.jsdelivr.net/gh/ColonelParrot/AnabolicSet@latest/src/anabolicset.min.js"></script>
```

## Snippets

### Basic Example

```javascript
const set = new AnabolicSet([1, 2, 3])
set.add(3)
set.add(4)
set.values() // [1, 2, 3, 4]
```
### Serializer

```javascript
const set = new AnabolicSet([], (obj) => obj.id)
set.addAll({ id: 1 }, { id: 2 }, { id: 2 })
set.values() // [{ id: 1 }, { id: 2 }]
```
**Note**: when adding objects, a serializer is required

### `union`, `intersect`, `complement`...

```javascript
const set1 = new AnabolicSet([1, 2, 3, 4, 5])
const set2 = new AnabolicSet([1, 2, 6])

set1.union(set2).values() // [1, 2, 3, 4, 5, 6]
set1.intersect(set2) // [1, 2]
set1.complement(set2) // [3, 4, 5]
set1.isSubsetOf(set2) // false
```

### 

## Methods

- [all default `Set` methods](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Set#instance_methods)
#### - `constructor(values, serializer, options)`
Set the initial values in set, serializer & options

```javascript
const set = new AnabolicSet([1, 2, 3])
set.values() // [1, 2, 3]

const set2 = new AnabolicSet([{ id: 1 }, { id: 2 }, { id: 2 }], (obj) => obj.id)
set2.values() // [{ id: 1 }, { id: 2 }]
```

#### - `addAll(...values)`
Adds multiple rest parameters

```javascript
set.addAll(1, 2, 3)
set.addAll(...[1, 2, 3])
```

#### - `setValues(values)`
Overwrite values with new values

```javascript
set.setValues([4, 5, 6]) // [4, 5, 6]
set.setValues(2) // [2]
```

#### - `setSerializer(serializer)`
Set the new serializer function (**will not reserialize values**)

```javascript
set.setSerializer((obj) => obj.id)
set.addAll({ id: 1 }, { id: 2 }, { id: 2 }) // [{ id: 1 }, { id: 2 }]
```

#### - `union(set)`
Returns `AnabolicSet`
Gets the [union](https://en.wikipedia.org/wiki/Union_(set_theory)) of two sets

```javascript
const set = new AnabolicSet([1, 2, 3])
const set2 = new AnabolicSet([3, 4, 5])
set.union(set2) // AnabolicSet [1, 2, 3, 4, 5]
```

#### - `intersect(set)`
Returns `Array` - gets the [intersection](https://en.wikipedia.org/wiki/Intersection_(set_theory)) of two sets

```javascript
set.intersect(set2) // [3]
```

#### - `complement(set)`
Returns `Array` - gets the [complement](https://en.wikipedia.org/wiki/Complement_(set_theory)) of two sets

```javascript
set.complement(set2) // [1, 2]
```

#### - `isSubsetOf(set)`
Returns `boolean` - checks if current set is a [subset](https://en.wikipedia.org/wiki/Subset) of parameter

```javascript
const set = new AnabolicSet([1, 2, 3, 4, 5])
const set2 = new AnabolicSet([4, 3, 2])
set2.isSubsetOf(set) // true
```

#### - `isSupersetOf(set)`
Returns `boolean` -  checks if current set is a superset of parameter

```javascript
set.isSupersetOf(set2) // true
```